# -*- coding: iso-8859-1 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3 (>= 3.7)
# Project  : Cedar Backup, release 3
# Purpose  : Provides location to maintain release information.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

"""
Provides location to maintain version information.

Module Attributes
=================

Attributes:
   AUTHOR: Author of software
   EMAIL: Email address of author
   COPYRIGHT: Copyright date
   VERSION: Software version
   DATE: Software release date
   URL: URL of Cedar Backup webpage

:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""

AUTHOR = "Kenneth J. Pronovici"
EMAIL = "pronovic@ieee.org"
COPYRIGHT = "2004-2020"
VERSION = "3.5.1"
DATE = "30 Dec 2020"
URL = "https://github.com/pronovic/cedar-backup3"
